package it.eng.spago.jbi;

import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.configuration.FileCreatorConfiguration;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.init.InitializerManager;
import it.eng.spago.jbi.component.ServiceUnitManagerProvider;
import it.eng.spago.jbi.init.JBIManager;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

public class ServiceUnitManager implements
		javax.jbi.component.ServiceUnitManager {

	private ComponentContext mContext = null;

	private ServiceUnitManagerProvider mProvider = null;

	public ServiceUnitManager(ComponentContext compCtx)
			throws DeploymentException {
		mContext = compCtx;

		try {
			Logger logger = mContext.getLogger(Constants.SPAGO_LOGGER, null);
			logger.log(Level.CONFIG, "ServiceUnitManager::ServiceUnitManager");

			String masterXMLPath = "\\WEB-INF\\conf\\spago\\master.xml";
			String installRoot = mContext.getInstallRoot();
			logger.log(Level.CONFIG, "ServiceUnitManager::installRoot=" + installRoot);
			logger.log(Level.CONFIG, "ServiceUnitManager::complete path=" + installRoot
					+ masterXMLPath);

			ConfigSingleton
					.setConfigurationCreation(new FileCreatorConfiguration());
			ConfigSingleton.setRootPath(installRoot);
			ConfigSingleton.setConfigFileName(masterXMLPath);
			ConfigSingleton.getInstance();
			logger.log(Level.CONFIG, ConfigSingleton.getInstance().toXML(
							false));

			InitializerManager.init();
		} catch (JBIException ex) {
			throw new DeploymentException(ex);
		}
	}

	public String deploy(String serviceUnitID, String serviceUnitRootPath)
			throws DeploymentException {
		try {
			mProvider = JBIManager.getInstance();
			mProvider.init(mContext);
			return mProvider.deploy(serviceUnitID, serviceUnitRootPath);
		} catch (EMFInternalError ex) {
			throw new DeploymentException(ex);
		}
	}

	public void init(String serviceUnitName, String serviceUnitRootPath)
			throws DeploymentException {
		try {
			Logger logger = mContext.getLogger(Constants.SPAGO_LOGGER, null);
			logger.log(Level.CONFIG, "ServiceUnitManager::init");
		} catch (JBIException ex) {
			throw new DeploymentException(ex);
		}
	}

	public void start(String serviceUnitName) throws DeploymentException {
		QName qn = new QName("http://spago.eng.it/spago.wsdl", mContext
				.getComponentName());
		try {
			Logger logger = mContext.getLogger(Constants.SPAGO_LOGGER, null);
			ServiceEndpoint ref = mContext
					.activateEndpoint(qn, "SpagoEndpoint");
			logger.log(Level.CONFIG, "SpagoEndpoint activated");
		} catch (JBIException JBIe) {
			JBIe.printStackTrace();
		}
	}

	public void stop(String arg0) throws DeploymentException {
		try {
			Logger logger = mContext.getLogger(Constants.SPAGO_LOGGER, null);
			logger.log(Level.CONFIG, "ServiceUnitManager::stop");
		} catch (JBIException ex) {
			throw new DeploymentException(ex);
		}
	}

	public void shutDown(String arg0) throws DeploymentException {
		try {
			Logger logger = mContext.getLogger(Constants.SPAGO_LOGGER, null);
			logger.log(Level.CONFIG, "ServiceUnitManager::shutDown");
		} catch (JBIException ex) {
			throw new DeploymentException(ex);
		}
	}

	public String undeploy(String arg0, String arg1) throws DeploymentException {
		try {
			Logger logger = mContext.getLogger(Constants.SPAGO_LOGGER, null);
			logger.log(Level.CONFIG, "ServiceUnitManager::undeploy");
			return null;
		} catch (JBIException ex) {
			throw new DeploymentException(ex);
		}
	}

}
